

 /*+-------------------------------------------------------+
   |  SAS Global Forum 2013 Sample                         |
   |  Google-like maps in SAS                              |          
   |                                                       |
   |    NAME: mapmacros.sas                                |
   |   TITLE: Misc Map macros used in examples             |
   | PRODUCT: GRAPH                                        |
   | PAPER TITLE:                                          |
   |   Google-like Maps in SAS                             |
   |                                                       |
   |                                                       |
   |18Jan2013                                              |
   +-------------------------------------------------------+*/



/***************************************************************
 * to_mercator -  Convert standard latitude and longitude to
 *               mercator projections for OpenStreet maps.  
 *               Input a data set with a list of points and 
 *               generate an output dataset with the points 
 *               converted to mecator projection.  If the points 
 *               are in variables named lat and long, then
 *               set latlong to 1.  Otherwise, set it to zero. 
 *               Converted points are always in x and y.
 *
 * Usage:      %to_mercator(outds,inds, latlong);
 *             example: %tomercator(test2,testout2,0);
 *
 * Arguments:  
 *             outds - output data set containing centroid  
 *                     values
 *             inds - input map data set
 *             latlog - 0,1. 1 means use long lat variable for x               
 *                      and yl.
 *                      Use 1 if the unprojected coordinates are  
 *                      stored in variable 'lat' and 'long'
 *                      instead of y and x.
 * Author: Darrell Massengill
 * Date:   11July2011
**************************************************************/

%macro to_mercator(outds,inds, latlong);
/* Convert to Mercator from degrees */
data &outds;
   set &inds;
   %if (&latlong=1) %then %do;
     /*then use the value stored in long/lat for the x/y */   
     /* values*/
     x=long;
     y=lat;
   %end;
   x= x * 20037508.34 / 180;
   y= log(tan((90+y) * constant('pi')/360)) / 
         (constant('pi') / 180);
   y= y * 20037508.34 / 180;
run;
%mend;

/*-----------------------------------------------------------*/
/* Macro to create point data in an annotate data set
 * annodata = point annotate data set created.  
 * inputdata = Input data set containing points to be drawn. 
 *      Note that this data must contain variables x and y 
 *      with the location of the points.
 * color = Graph color.  eg: red
 * size = dot size (optional).  Default is .7;
 * tipvar = Variable name to be used for tool tip (optional).
 * 
 */    
%macro make_dots(annodata, inputdata, color, size, tipvar); 
data &annodata;
  length function color $ 8 style $20 position $ 1 text $ 60 html $1024;
  retain xsys ysys '2' hsys '3' when 'a' size .7;
  set &inputdata;
  /*optional arguments*/
  %if (&size ^= ) %then %do;
    size=&size;
  %end;
  /* All annotate points are 360-degree pies.  */
  function='pie';
  rotate=360;
  style='psolid';
  color=&color;

  /* optional tool tips */
  %if (&tipvar ^= ) %then %do;
    html= 'title=' || quote("&tipvar" ||'=' || 
           trim(left(&tipvar)) );
  %end;


  output;
  /* Draw an outline around each 'dot' */
  color="gray55"; style='pempty';
  output;
run;
%mend;
/*-----------------------------------------------------------*/


/*-----------------------------------------------------------*/
/* Macro to convert degrees radians to degrees.
 * outdata = output data that is created. 
 * inputdata = Input data set containing points to be drawn. 
 *      Note that this data must contain variables x and y 
 *      with the location of the points.
 */    
%macro to_degrees(outdata, inputdata);
/* Convert from radians to degrees */
data &outdata;
  set &inputdata;
  x=-x*45/atan(1);
  y= y*45/atan(1);
run;
%mend;
/*-----------------------------------------------------------*/

/*-----------------------------------------------------------*/
/* Macro to convert degrees to radians.
 * outdata = output data that is created. 
 * inputdata = Input data set containing points to be drawn. 
 *      Note that this data must contain variables x and y 
 *      with the location of the points.
 */    
%macro to_radians(outdata, inputdata);
/* Convert from degrees to radians */
data &outdata;
  set &inputdata;
  x=-x*atan(1)/45;
  y= y*atan(1)/45;
run;
%mend;
/*-----------------------------------------------------------*/




/*-----------------------------------------------------------*/
/* Macro to create a circle as a polygon in annotate datasets
 * outdata = circle map data set to be created (annotate or map)
 * inputdata = optional input data set containing center 
 *             locations 
 * centerX = x position of center of circle (VAR/CON)
 * centerY = y position of center of circle (VAR/CON)
 * miles = radius of circle in miles.  From center to circle
 *         (VAR/CON)
 * color = Graph color.  (VAR/CON)
 * size  = Size of line.  Optional.  Default is 3. (VAR/CON)
 * 
 * Usage:  Inputdata is optional.  If not specified, centerX,
 *         centerY and miles must be specified as constants.
 *         If specified, centerY, centerX and miles can be 
 *         the name of the variable in the inputdata containing 
 *         the values for each.  Or it can be a constant.
 */       
%macro make_circle(outdata,inputdata, centerx, centery, miles, 
                   color, size);
data &outdata;
  retain id 0;
  length text $25 color function $8 style $20;
  retain xsys ysys '2' hsys '3' anno_flag 2 
         size 3 when 'a' text ' ';
  /*handle optional arguments*/
  %if (&inputdata ^= ) %then %do;
    set &inputdata;
  %end;
  %if (&size ^= ) %then %do;
    size=&size;
  %end;

  /* Degree to radian constant */
  d2r=atan(1)/45;

  /* Miles per radian constant */
  r2m=3958.739565;

  /* Save off the X/Y for the school (first obs) to be used */
  /* as the center point */
  xcen=&centerx;
  ycen=&centery;

  /* ID allows this same data set to be used as a MAP rather */
  /* than just annotate */
  id=id+1;
  /* Calculate the points to draw a circle */
  /* This must be done as a polygon rather than a pie function 
     in order to maintain the correct distance after projecting.
     It's quite possible that the circle is no longer a circle
     but more of an ellipse once we project */
  do degree=0 to 365 by 5;  

     /* Begin a new circle */
     if degree=0 then do;

        /* The circles will be solid */
        function='poly';
        style='empty';
        line=1;
        end;

     /* Continue drawing the circle */
     else do;

        /* Outline the circle with desired color */
        function='polycont';
        color=&color;
        end;

     /* Calculate a point along the circle */
     y=arsin(cos(degree*d2r)*sin(&miles/r2m)*cos(ycen)
         +cos(&miles/r2m)*sin(ycen));
     x=xcen+arsin(sin(degree*d2r)*sin(&miles/r2m)/cos(y));
     output;
  end;
  drop degree r2m d2r xcen ycen;
run;
%mend;
/*-----------------------------------------------------------*/


/*-----------------------------------------------------------*/
/* Macro to display a map.
 * mapdata = map data set.
 * respdata = response data set.
 * annodata = annotate data set.
 * id = id statement variable.
 * choro = choro statement variable.
 * name = file name to output.
 * dev = device to draw the map.
 * mapcolor = "Y" to color the map background or "N" to not 
 *            color 
 *            map background (optional - default is "Y"). 
 *            "C" means to set to the set of Choropleth colors.
 * htmlvar = Variable containing html info.  (optional)
 */ 
%macro draw_map(mapdata,respdata, annodata, id, choro, name,  
                dev,  mapcolor, htmlvar);
   /*******************************************************/
   /* Create an HTML file showing the map and points      */
   /*******************************************************/
   *GOPTIONS reset=all;
   GOPTIONS CBACK=white;
   GOPTIONS DEVICE=&dev;
   ODS LISTING CLOSE;
   ODS HTML path=odsout body="&name._&dev..html"
    ;
   pattern;

   %if ((&mapcolor eq ) or (&mapcolor = "Y")) %then %do;
     pattern v=s c=CXE9E8DC r=100;
   %end;
   %else %if ((&mapcolor = "N")) %then %do;
     *pattern v=me c=CX000000 r=100;
      pattern v=me c=A00000000 r=100;
   %end; 
   %else %do; /*Choropleth colors*/
      pattern1  v=solid repeat=1 c=A00ff00aa;
      pattern2  v=solid repeat=1 c=A0000feaa;
      pattern3  v=solid repeat=1 c=A3661feaa;
      pattern4  v=solid repeat=1 c=A38acffaa;
      pattern5  v=solid repeat=1 c=A00ffffaa;
      pattern6  v=solid repeat=1 c=A91ffb4aa;
      pattern7  v=solid repeat=1 c=Ad2fe69aa;
      pattern8  v=solid repeat=1 c=Affff00aa;
      pattern9  v=solid repeat=1 c=Affb700aa;
      pattern10 v=solid repeat=1 c=Aff6f00aa;
      pattern11 v=solid repeat=1 c=Afe0000aa;
   %end;
   %if (&htmlvar ^= ) %then %do;
      %let htmlv=html=&htmlvar;
   %end;
   %else %do;
      %let htmlv=;
   %end;
   /* this enables the OSM maps for JAVA* devices */
   %if ((&dev = JAVA) or (&dev=JAVAIMG)) %then %do;
      %let showosm=showosm;
   %end;
   %else %do;
      %let showosm=;
   %end;

   goptions border;

   proc gmap map=&mapdata anno=&annodata data=&respdata /*all*/;  
        id &id;  
        choro &choro /nolegend &showosm name="&name" &htmlv;
   run;
   quit;
   ODS HTML CLOSE;
   ODS LISTING;
%mend;

